﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Runtime.InteropServices;    // For StructLayout, DllImport
using WhaleTeqPPG_SDK;


namespace WhaleteqPPG_SDK_Demo
{
    public partial class WhaleteqPPG_SDK_Demo : Form
    {
        public delegate void ConnectCBDelegate(bool connected);
        private static ConnectCBDelegate fpConnect; //avoid CB be GC

        public delegate void SamplingCBDelegate(int valueNum, IntPtr valueArray);
        private static SamplingCBDelegate fpSampling; //avoid CB be GC

        public delegate void PeakCBDelegate(int maxPeak);
        private static PeakCBDelegate fpPeak; //avoid CB be GC
        private IntPtr _ipPeakCB;

        private PDSamplingGraph mSamplingZ = null;

        public WhaleteqPPG_SDK_Demo()
        {
            InitializeComponent();

            //Initialize SDK
            fpConnect = new ConnectCBDelegate(ConnectCB);
            IntPtr ipConnect = Marshal.GetFunctionPointerForDelegate(fpConnect);

            try
            {
                bool RV = PPG_SDK.InitPPG(ipConnect);
                updateConnect(RV);
            }
            catch (Exception e)
            {
                MessageBox.Show(e.ToString());
                return;
            }

            //Register Sampling CB
            fpSampling = new SamplingCBDelegate(PDSamplingCB);
            IntPtr ipSamp = Marshal.GetFunctionPointerForDelegate(fpSampling);
            PPG_SDK.RegisterPDSamplingCB(ipSamp);

            //Register Peak CB
            fpPeak = new PeakCBDelegate(PeakCB);
            _ipPeakCB = Marshal.GetFunctionPointerForDelegate(fpPeak);
            PPG_SDK.RegisterPDSamplingPeakCB(_ipPeakCB, (double)numericUpDown_PeakCycle.Value / 1000);

            //Initialize PD-Sampling Graph
            mSamplingZ = new PDSamplingGraph();
            panel_GraphPDSampling.Controls.Add(mSamplingZ);
            Size size = panel_GraphPDSampling.Size;
            mSamplingZ.Size = size;
        }

        private void ConnectCB(bool connected)
        {
            updateConnect(connected);
        }

        private void updateConnect(bool connected)
        {
            string title = "WhaleTeq HRS SDK Demo";
            if (connected)
            {
                Text = title + " - Device Connected";
                groupBox_Scale.Enabled = PPG_SDK.IsWiderPDRange();
            }
            else
                Text = title + " - Device Unconnected";

        }

        private void WhaleteqPPG_SDK_Demo_FormClosed(object sender, FormClosedEventArgs e)
        {
            PPG_SDK.ClosePPG();
        }


        //PD Sampling ------------------------------------
        private void btn_PDSampling_Start_Click(object sender, EventArgs e)
        {
            PPG_SDK.SetSampleSetting(true, false);

            btn_PDSampling_Start.Enabled = false;
            btn_PDSampling_Stop.Enabled = true;
        }

        private void btn_PDSampling_Stop_Click(object sender, EventArgs e)
        {
            PPG_SDK.SetSampleSetting(false, false);

            btn_PDSampling_Start.Enabled = true;
            btn_PDSampling_Stop.Enabled = false;

            label_PeakMax.Text = "";
        }

        private void numericUpDown_PeakCycle_ValueChanged(object sender, EventArgs e)
        {
            PPG_SDK.RegisterPDSamplingPeakCB(_ipPeakCB, (double)numericUpDown_PeakCycle.Value / 1000);
        }

        private void panel_GraphPDSampling_Resize(object sender, EventArgs e)
        {
            Size size = new Size(panel_GraphPDSampling.Width, panel_GraphPDSampling.Height);
            if (null != mSamplingZ)
                mSamplingZ.Size = size;
        }

        private void PDSamplingCB(int valueNum, IntPtr valueArray)
        {
            if (!checkBox_Graph.Checked)
                return;

            short[] dataArray = new short[valueNum];
            Marshal.Copy(valueArray, dataArray, 0, valueNum);

            mSamplingZ.Sampling(dataArray);
        }

        private void PeakCB(int maxPeak)
        {
            if (!checkBox_Peak.Checked)
                return;

            label_PeakMax.Text = maxPeak.ToString();
        }

        private void radioButton_1000_CheckedChanged(object sender, EventArgs e)
        {
            RadioButton RB = (RadioButton)sender;
            if (!RB.Checked)
                return;

            PPG_SDK.SetSampleSetting(false, false);
            PPG_SDK.SetSampleChannel(radioButton_10000.Checked ? 1 : 0);

            //int nMax = radioButton_10000.Checked ? 10000 : 1000;
            //mSamplingZ.UpdateScale(nMax, 0);

            if (checkBox_LedOff.Checked)
                PPG_SDK.SendLedOff();

            PPG_SDK.SetSampleSetting(true, checkBox_LedOff.Checked);
        }

        //LED Command ------------------------------------
        const float MIN_AC = 75.0f;

        private bool mInternalUpdate = false;

        private void updatedHRS()
        {
            int dc = (int)numDCLevel.Value;
            float pi = (float)numPIValue.Value;
            int bpm = (int)numBPM.Value;

            PPG_SDK.SendCommand(dc, pi, bpm, checkBox_SyncAuto.Checked);
        }

        private void HRS_DCChanged(object sender, EventArgs e)
        {
            int lvl = Convert.ToInt32(numDCLevel.Value);
            float vb = 2500.0f * lvl / 20;

            float maxPI = 2500.0f / vb;
            float minPI = MIN_AC / vb;
            if (minPI < 0.1f)
                minPI = 0.1f;

            mInternalUpdate = true;
            numPIValue.Maximum = (decimal)maxPI;
            numPIValue.Minimum = (decimal)minPI;
            mInternalUpdate = false;

            labPI.Text = "PI (" + minPI.ToString("0.0") + "~" + maxPI.ToString("0.0") + "%)";

            if (mInternalUpdate)
                return;

            updatedHRS();
        }

        private void HRS_ValueChanged(object sender, EventArgs e)
        {
            if (mInternalUpdate)
                return;

            updatedHRS();
        }

        private void checkBox_Sync_CheckedChanged(object sender, EventArgs e)
        {
            updatedHRS();
        }

        private void checkBox_LedOff_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox_LedOff.Checked)
            {
                if (btn_PDSampling_Stop.Enabled) // Is PDSampling
                    PPG_SDK.SetSampleSetting(false, false); //Stop Sampling

                PPG_SDK.SendLedOff();

                if (btn_PDSampling_Stop.Enabled) // Is PDSampling
                    PPG_SDK.SetSampleSetting(true, true); //Start High-Resolution Sampling
            }
            else
            {
                if (btn_PDSampling_Stop.Enabled) // Is PDSampling
                    PPG_SDK.SetSampleSetting(false, false); //Stop Sampling

                updatedHRS();

                if (btn_PDSampling_Stop.Enabled) // Is PDSampling
                    PPG_SDK.SetSampleSetting(true, false); //Start Low-Resolution Sampling
            }
        }
    }
}
