﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;    // For StructLayout, DllImport

using WhaleTeqPPG_SDK;

namespace WhaleteqPPG_SDK_FineTune
{
    public partial class MainForm : Form
    {
        public delegate void ConnectCBDelegate(bool connected);
        private static ConnectCBDelegate fpConnect; //avoid CB be GC

        private Graph mManualZ = null;

        public MainForm()
        {
            InitializeComponent();

            //Initialize FineTune Graph
            mManualZ = new Graph();
            palGraphManual.Controls.Add(mManualZ);
            Size size = palGraphManual.Size;
            mManualZ.Size = size;

            //Initialize SDK
            fpConnect = new ConnectCBDelegate(ConnectCB);
            IntPtr ipConnect = Marshal.GetFunctionPointerForDelegate(fpConnect);

            try
            {
                bool RV = PPG_SDK.InitPPG(ipConnect);
                updateConnect(RV);
            }
            catch (Exception e)
            {
                MessageBox.Show(e.ToString());
                return;
            }
        }

        private void ConnectCB(bool connected)
        {
            updateConnect(connected);
        }

        private void updateConnect(bool connected)
        {
            string title = "Whaleteq HRS SDK - Fine Tune Demo";
            if (connected)
            {
                Text = title + " - Device Connected";
                updatedManual();
            }
            else
                Text = title + " - Device Unconnected";
        }

        private void WhaleteqPPG_SDK_FineTune_FormClosed(object sender, FormClosedEventArgs e)
        {
            PPG_SDK.ClosePPG();
        }

        // Fine Tune -----------------------------------------------------------
        private void updatedManual()
        {
            int dc = (int)numDC.Value;
            float fAC = (float)numAC.Value;
            int ac = (int)(fAC * 100);
            int bpm = (int)numBPM_M.Value;

            OutputFunction wave = OutputFunction.Output_Sine;
            switch (comboBox_Wave.SelectedIndex)
            {
                case 0:
                    wave = OutputFunction.Output_Sine;
                    break;
                case 1:
                    wave = OutputFunction.Output_Triangle;
                    break;
                case 2:
                    wave = OutputFunction.Output_PPG;
                    break;
            }

            int noiseHz = 0;
            switch (comboBox_Noise.SelectedIndex)
            {
                case 1:
                    noiseHz = 50;
                    break;
                case 2:
                    noiseHz = 60;
                    break;
                case 3:
                    noiseHz = 1000;
                    break;
                case 4:
                    noiseHz = 5000;
                    break;
            }

            if (null != mManualZ)
            {
                mManualZ.RegisterSDK();
            }

            PPG_SDK.SendFineTune(wave, noiseHz, dc, ac, bpm, checkBox_SyncManual.Checked);
        }

        private void manual_DCChanged(object sender, EventArgs e)
        {
            int vb = (int)numDC.Value;
            labVbManual.Text = "Vb = " + vb.ToString() + "mV";

            float pi = (float)numAC.Value * 100 / vb;
            labPiManual.Text = "PI = " + pi.ToString("0.00") + "%";

            updatedManual();
        }

        private void manual_ACChanged(object sender, EventArgs e)
        {
            float ac = (float)numAC.Value;
            labVaManual.Text = "Va =" + ac.ToString("0.00") + "mV";

            float pi = ac * 100 / (int)numDC.Value;
            labPiManual.Text = "PI = " + pi.ToString("0.00") + "%";

            updatedManual();
        }

        private void manual_HRChanged(object sender, EventArgs e)
        {
            updatedManual();
        }

        private void comboBox_Wave_SelectedIndexChanged(object sender, EventArgs e)
        {
            updatedManual();
        }

        private void comboBox_Noise_SelectedIndexChanged(object sender, EventArgs e)
        {
            numericUpDown_NoiseAmp.Enabled = (comboBox_Noise.SelectedIndex != 0);
            updatedManual();
        }

        private void numericUpDown_NoiseAmp_ValueChanged(object sender, EventArgs e)
        {
            double amp = (double)numericUpDown_NoiseAmp.Value;
            PPG_SDK.SendFineTuneNoiseAmp(amp);
        }

        private void palGraphManual_Resize(object sender, EventArgs e)
        {
            if (null != mManualZ)
            {
                Size size = new Size(palGraphManual.Width, palGraphManual.Height);
                mManualZ.Size = size;
            }
        }



    }
}
