﻿//#define THREAD_SENDING

using System;
using System.Collections.Generic;
//using System.Linq;
using System.Text;
//using System.Threading.Tasks;

using System.Drawing;

using System.Runtime.InteropServices;    // For StructLayout, DllImport
using ZedGraph;
using WhaleTeqPPG_SDK;

namespace WhaleteqPPG_SDK_PlayRawData
{
    public class DynamicGraph : ZedGraphControl
    {
        public enum TimeUnit_E
        {
            SECOND,
            MINI_SECOND,
            MICRO_SECOND
        };

        private double _timeMax;
        private TimeUnit_E _timeUnit;

        public DynamicGraph(double timeMax = 5, TimeUnit_E unit = TimeUnit_E.SECOND)
        {
            _timeMax = timeMax;
            _timeUnit = unit;

            ScrollGrace = 0;
            ScrollMaxX = 0;
            ScrollMaxY = 0;
            ScrollMaxY2 = 0;
            ScrollMinX = 0;
            ScrollMinY = 0;
            ScrollMinY2 = 0;

            GraphPane GP = GraphPane;
            LineItem M = GP.AddCurve("", new PointPairList(), Color.Blue, SymbolType.None);
            LineItem N = GP.AddCurve("", new PointPairList(), Color.DarkViolet, SymbolType.None);

            GP.Title.Text = "File";
            GP.YAxis.Title.Text = "Value";

            if (_timeUnit == TimeUnit_E.SECOND)
                GP.XAxis.Title.Text = "Time (s)";
            else if (_timeUnit == TimeUnit_E.MINI_SECOND)
                GP.XAxis.Title.Text = "Time (ms)";
            else if (_timeUnit == TimeUnit_E.MICRO_SECOND)
                GP.XAxis.Title.Text = "Time (ms)";

            Axis X = GP.XAxis;
            X.Scale.Max = _timeMax;
            X.Scale.Min = 0;
            X.Scale.MajorStep = _timeMax / 5;
            X.Scale.MinorStep = X.Scale.MajorStep / 5;
            X.MinorGrid.IsVisible = true;
            X.MinorGrid.DashOff = 1;
            X.MinorGrid.DashOn = 5;
            X.MinorGrid.Color = Color.Pink;
            X.MajorGrid.IsVisible = true;
            X.MajorGrid.DashOff = 0;
            X.MajorGrid.DashOn = 10;
            X.MajorGrid.Color = Color.Pink;

            Axis Y = GP.YAxis;
            Y.MinorGrid.IsVisible = true;
            Y.MajorGrid.IsVisible = true;
            Y.MinorGrid.Color = Color.Pink;
            Y.MajorGrid.Color = Color.Pink;
            Y.IsAxisSegmentVisible = false;

            IsEnableVPan = false;
            IsEnableVZoom = false;
            IsEnableWheelZoom = false;
            IsEnableZoom = false;
        }

        public void UpdateScale(int bits)
        {
            Axis Y = GraphPane.YAxis;

            Y.Scale.Max = (1 << bits);
            Y.Scale.Min = 0;
            Y.Scale.MajorStep = (int)(1 << bits) / 4;
            Y.Scale.MinorStep = Y.Scale.MajorStep / 2;

            GraphPane.CurveList[0].Clear();
            GraphPane.CurveList[1].Clear();

            Invalidate();
        }

        public void UpdateScale(int max, int min)
        {
            Axis Y = GraphPane.YAxis;

            Y.Scale.Max = max;
            Y.Scale.Min = min;
            Y.Scale.MajorStep = max / 4;
            Y.Scale.MinorStep = Y.Scale.MajorStep / 2;

            GraphPane.CurveList[0].Clear();
            GraphPane.CurveList[1].Clear();

            Invalidate();
        }

        public void ResetGraph()
        {
            GraphPane.CurveList[0].Clear();
            GraphPane.CurveList[1].Clear();
        }

        private const int ZBufSize = 128;
        private double[] ZtgBuf = new double[ZBufSize];
        private short[] ZampBuf = new short[ZBufSize];
        private short[] ZampBuf2 = new short[ZBufSize];
        private int ZBufIdx = 0;
        private int ZResetCountdown = 0;
        private double lastTg = 0;
        public bool TimeAmplitudeCB(double time, short amp1, short amp2)
        {
            bool RV = false;

            double tg = time - ((int)time / 5) * 5;

            if (0 != ZResetCountdown)
            {
                if (0 == --ZResetCountdown)
                    ResetGraph();
            }

            if (ZBufIdx == ZBufSize || tg < lastTg)
            {
                if (tg < lastTg)
                {
                    ZResetCountdown = (ZBufSize / 2);

                    RV = true;
                }

                for (int i = 0; i < ZBufIdx; i++)
                {
                    GraphPane.CurveList[0].AddPoint(ZtgBuf[i], ZampBuf[i]);
                    GraphPane.CurveList[1].AddPoint(ZtgBuf[i], ZampBuf2[i]);
                }

                Invalidate();
                ZBufIdx = 0;
            }

            ZtgBuf[ZBufIdx] = tg;
            ZampBuf[ZBufIdx] = amp1;
            ZampBuf2[ZBufIdx++] = amp2;

            lastTg = tg;

            return RV;
        }

        //For Sampling -------------------------------------------------------
        private double _SamplingTime = 0.03;

        public void SetSampleTime(double time)
        {
            _SamplingTime = time;

            double dPixelTime = _timeMax / Width;
            _PerSample = (int)(dPixelTime / time) + 1;

            _CurrPerSample = 0;
        }

        int _CurrPerSample = 0;
        int _PerSample;
        double s_tg = 0;
        public void Sampling(short amp, ref bool frameEnd)
        {
            s_tg += _SamplingTime;

            if (++_CurrPerSample < _PerSample)
                return;
            _CurrPerSample = 0;

            if (0 != ZResetCountdown)
            {
                if (0 == --ZResetCountdown)
                    ResetGraph();
            }

            if (ZBufIdx == ZBufSize || s_tg > _timeMax)
            {
                if (s_tg > _timeMax)
                {
                    ZResetCountdown = (ZBufSize / 2);
                    s_tg = 0;

                    frameEnd = true;
                }

                for (int i = 0; i < ZBufIdx; i++)
                    GraphPane.CurveList[0].AddPoint(ZtgBuf[i], ZampBuf[i]);

                Invalidate();
                ZBufIdx = 0;
            }

            ZtgBuf[ZBufIdx] = s_tg;
            ZampBuf[ZBufIdx++] = amp;
        }

        public void SetUnit(double timeMax, TimeUnit_E unit)
        {
            _timeMax = timeMax;
            _timeUnit = unit;

            GraphPane GP = GraphPane;
            if (_timeUnit == TimeUnit_E.SECOND)
                GP.XAxis.Title.Text = "Time (s)";
            else if (_timeUnit == TimeUnit_E.MINI_SECOND)
                GP.XAxis.Title.Text = "Time (ms)";
            else if (_timeUnit == TimeUnit_E.MICRO_SECOND)
                GP.XAxis.Title.Text = "Time (ms)";

            Axis X = GP.XAxis;
            X.Scale.Max = _timeMax;
            X.Scale.MajorStep = _timeMax / 5;
            X.Scale.MinorStep = X.Scale.MajorStep / 5;

            GraphPane.CurveList[0].Clear();

            s_tg = 0;
            ZBufIdx = 0;

            Invalidate();
        }
    }
}
