﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;    // For StructLayout, DllImport

using WhaleTeqPPG_SDK;

namespace WhaleteqPPG_SDK_PlayRawData
{
    public partial class MainForm : Form
    {
        public delegate void ConnectCBDelegate(bool connected);
        private static ConnectCBDelegate fpConnect; //avoid CB be GC

        private LoadGraph mDynamicZ = null;
        private DynamicGraph mPlayingZ = null;

        public delegate void PlayStopCBDelegate();
        private static PlayStopCBDelegate fpDynamicStop; //avoid CB be GC

        public delegate void DynamicPlayCBDelegate(double time, short dc, short ac);
        private static DynamicPlayCBDelegate fpDynamicTimeAmp; //avoid CB be GC


        private double mDynamicStopTime = 0.0;
        private double mDynamicPlayTime = 0.0;

        public MainForm()
        {
            InitializeComponent();

            //Initialize SDK
            fpConnect = new ConnectCBDelegate(ConnectCB);
            IntPtr ipConnect = Marshal.GetFunctionPointerForDelegate(fpConnect);

            try
            {
                bool RV = PPG_SDK.InitPPG(ipConnect);
                updateConnect(RV);
            }
            catch (Exception e)
            {
                MessageBox.Show(e.ToString());
                return;
            }

            //Initialize PlayRawData Graph
            mDynamicZ = new LoadGraph();
            palGraphDynamic.Controls.Add(mDynamicZ);
            Size size = palGraphDynamic.Size;
            mDynamicZ.Size = size;

            mPlayingZ = new DynamicGraph();
            mPlayingZ.UpdateScale(5000, 0);
            mPlayingZ.GraphPane.Title.Text = "PPG Output";
            palGraphPlaying.Controls.Add(mPlayingZ);
            size = palGraphPlaying.Size;
            mPlayingZ.Size = size;
        }

        private void ConnectCB(bool connected)
        {
            updateConnect(connected);
        }

        private void updateConnect(bool connected)
        {
            string title = "Whaleteq HRS SDK - Play Raw Data Demo";
            if (connected)
                Text = title + " - Device Connected";
            else
                Text = title + " - Device Unconnected";
        }

        private void btnDynamicLoad_Click(object sender, EventArgs e)
        {
            OpenFileDialog FD = new OpenFileDialog();
            FD.FileName = "*.txt";
            if (FD.ShowDialog() == DialogResult.OK)
            {
                LoadDynamic(FD.FileName);
            }
        }

        private void btnDynamicPlay_Click(object sender, EventArgs e)
        {
            if (mDynamicStopTime == 0.0)
                return;

            mDynamicPlayTime = 0.0;

            if (null == fpDynamicTimeAmp)
                fpDynamicTimeAmp = new DynamicPlayCBDelegate(DynamicTimeAmplitudeCB);
            IntPtr ipTimeAmp = Marshal.GetFunctionPointerForDelegate(fpDynamicTimeAmp);

            IntPtr ipPlayStop;
            if (chkBoxLoop2.Checked) //SDK default support "Loop Mode"
            {
                ipPlayStop = (IntPtr)0;
            }
            else
            {
                if (null == fpDynamicStop)
                    fpDynamicStop = new PlayStopCBDelegate(DynamicStopCB);
                ipPlayStop = Marshal.GetFunctionPointerForDelegate(fpDynamicStop);
            }

            double F = (double)numDynamicF.Value;
            int Offset = (int)numDynamicOffset.Value;
            if (!PPG_SDK.PlayDynamic(F, Offset, ipPlayStop, ipTimeAmp, checkBox_SyncRawData.Checked))
            {
                MessageBox.Show("Play Dynamic File Failed!");
                return;
            }

            btnDynamicLoad.Enabled = false;
            btnDynamicPlay.Enabled = false;
            btnDynamicStop.Enabled = true;

            chkBoxLoop2.Enabled = false;
            numDynamicF.Enabled = false;
            numDynamicOffset.Enabled = false;
            numDynamicSR.Enabled = false;
        }

        private void btnDynamicStop_Click(object sender, EventArgs e)
        {
            PPG_SDK.SendLedOff();

            btnDynamicLoad.Enabled = true;
            btnDynamicPlay.Enabled = true;
            btnDynamicStop.Enabled = false;

            chkBoxLoop2.Enabled = true;
            numDynamicF.Enabled = true;
            numDynamicOffset.Enabled = true;
            numDynamicSR.Enabled = true;

            mDynamicZ.UpdateTime(0.0d);
            label_RawDataTime.Text = mDynamicStopTime.ToString("0.0");
        }

        private void checkBox_Sync_CheckedChanged(object sender, EventArgs e)
        {
            if (!btnDynamicLoad.Enabled) //IsPlaying
            {
                PPG_SDK.SendLedOff();
                btnDynamicPlay_Click(null, null);
            }
        }

        private void numDynamicSR_ValueChanged(object sender, EventArgs e)
        {
            double newRate = (double)numDynamicSR.Value;

            PPG_SDK.ResampleDynamic(newRate, ref mDynamicStopTime);
            label_RawDataTime.Text = mDynamicStopTime.ToString("0.0");

            int num = 0;
            IntPtr ptr = PPG_SDK.GetDynamicArray(ref num);
            double[] result = new double[num];
            Marshal.Copy(ptr, result, 0, num);

            mDynamicZ.UpdateDynamic(0.0, 0.0, newRate, result);
        }

        private void palGraphDynamic_Resize(object sender, EventArgs e)
        {
            if (null != mDynamicZ)
            {
                Size size = new Size(palGraphDynamic.Width, palGraphDynamic.Height);
                mDynamicZ.Size = size;
            }
        }

        private void palGraphPlaying_Resize(object sender, EventArgs e)
        {
            if (null != mPlayingZ)
            {
                Size size = new Size(palGraphPlaying.Width, palGraphPlaying.Height);
                mPlayingZ.Size = size;
            }
        }


        private bool LoadDynamic(string file)
        {
            double rate = 0;
            int num = 0;
            double max = 0, min = 0;
            double stopTime = 0;
            num = PPG_SDK.LoadDynamic(file, ref rate, ref max, ref min, ref stopTime);
            if (num <= 0)
            {
                string errMsg = "";
                if (num == 0)
                {
                    errMsg = "Sample data too few: " + num.ToString();
                }
                else if (num == -1)
                {
                    errMsg = "Open file failed!";
                }
                else if (num == -2)
                {
                    errMsg = "Can't solve sample rate in Line 1!";
                }
                else if (num == -3)
                {
                    errMsg = "Can't solve sample number in Line 2!";
                }
                else if (num == -4)
                {
                    errMsg = "Raw data file is too large to fit the memory!";
                }
                MessageBox.Show(errMsg);

                return false;
            }

            LoadSetUI(max, min, rate, stopTime, file);

            return true;
        }

        private void LoadSetUI(double max, double min, double rate, double stopTime, string graphTitle)
        {
            int num = 0;
            IntPtr ptr = PPG_SDK.GetDynamicArray(ref num);
            double[] result = new double[num];
            Marshal.Copy(ptr, result, 0, num);

            label_RawDataTime.Text = stopTime.ToString("0.0");

            mDynamicZ.UpdateDynamic(max, min, rate, result);
            mDynamicZ.GraphPane.Title.Text = graphTitle;

            mPlayingZ.ResetGraph();
            mPlayingZ.Invalidate();

            mDynamicStopTime = stopTime;

            btnDynamicPlay.Enabled = true;

            chkBoxLoop2.Enabled = true;

            groupBox_RawDataParam.Enabled = true;

            double dDefault = 4095.0 / (max - min);
            numDynamicF.Maximum = (decimal)(dDefault * 4);
            numDynamicF.Value = (decimal)dDefault;

            numDynamicOffset.Minimum = (decimal)(-min);
            numDynamicOffset.Maximum = (decimal)(max);
            numDynamicOffset.Value = (decimal)(-min);
            numDynamicSR.Value = (decimal)rate;
        }

        private void DynamicTimeAmplitudeCB(double time, short dc, short ac)
        {
            bool updateTime = (int)(time * 10) != (int)(mDynamicPlayTime * 10);

            bool updateGraph = (((int)time) / 5) != (((int)mDynamicPlayTime) / 5);

            mDynamicPlayTime = time;

            if (updateTime)
                label_RawDataTime.Text = mDynamicPlayTime.ToString("0.0");

            if (updateGraph)
                mDynamicZ.UpdateTime(time);

            mPlayingZ.TimeAmplitudeCB(time, dc, ac);
        }

        private void DynamicStopCB()
        {
            btnDynamicStop_Click(null, null);
        }

    }
}
